<?php
/*
Plugin Name: Easy Admin Color Schemes
Plugin URI: http://www.jamesdimick.com/creations/easy-admin-color-schemes/
Description: The Easy Admin Color Schemes plugin allows users to easily customize the colors of the administration interface for WordPress. It works by adding a new page to the Settings area in the WordPress admin interface. Users can use the simple form to change the look of the admin interface without needing to know a great deal about WordPress. The plugin generates a new stylesheet file for each color scheme created. This allows for seamless integration with the color scheme functions already available in the WordPress 2.5+ core.
Version: 2.6
Author: James Dimick
Author URI: http://www.jamesdimick.com/

=== VERSION HISTORY ===
  04.18.08 - v1.0 - The first version
  04.20.08 - v1.1 - Changed some URL query variable names in an attempt to prevent clashing with other plugins
  04.21.08 - v1.2 - Changed the way Last Modified dates are handled so the plugin still works instead of erroring out
  04.22.08 - v1.3 - Fixed an issue with slashes in the CSS content when you save a scheme
  04.23.08 - v1.4 - Added a better-looking default color scheme called Washedout
  04.23.08 - v1.5 - Added error codes to aid in debugging and fixed some image issues with the new Washedout color scheme
  04.26.08 - v1.6 - Added a link to the user profile page right by the Admin Color Scheme selector which links to the plugin
  04.26.08 - v1.7 - Fixed the way the new link on the profile page works so only users with proper permissions can see it
  04.29.08 - v1.8 - Corrected some things to allow for better localization
  05.09.08 - v2.0 - Added a new export feature which allows users to export color schemes in a couple different formats
                  |- Also added an upload feature so color schemes that have been exported can be imported back in
                  |- Fixed some issues with setting the current scheme from the plugin page
                  |- Fixed a few spelling errors in various parts of the plugin
                  |- Fixed some problems with the JavaScript
                  |- Fixed a few issues with localization
                  |- Improved the error reporting functions
  05.14.08 - v2.1 - Fixed the major issue some people were having with setting their current scheme from the plugin page
                  |- As a positive side-effect, setting the scheme from the built-in scheme picker on the profile page now works correctly
  06.25.08 - v2.2 - Removed the link on the user profile page until it can be done more reliably
                  |- Also added a new scheme called For the Love of 2.3 which attempts to bring back some of the old WordPress 2.3 styles
  06.28.08 - v2.3 - Fixed an issue with the new For the Love of 2.3 color scheme
                  |- Moved the update preview button to just above the preview window
                  |- Added a note just below the primary colors area in an attempt to relieve some confusion many have been having
                  |- Updated the included screenshot to include the new changes
  06.28.08 - v2.4 - Fixed (hopefully) the issue with the last modified dates
  08.18.08 - v2.5 - Fixed some issues caused by WordPress 2.6 including an issue with the For the Love of 2.3 scheme
                  |- Also updated the Washedout color scheme a bit
  08.21.08 - v2.6 - Fixed an issue with exporting color schemes with Method 2 of the export functionality
                  |- Fixed a few minor issues with the localization parts of the plugin
                  |- Also added a small bit to the Right Now section on the dashboard

=== LEGAL INFORMATION ===
  Copyright (C) 2008 James Dimick <mail@jamesdimick.com> - www.jamesdimick.com

  This program is free software: you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation, either version 3 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program. If not, see <http://www.gnu.org/licenses/>.
*/

$plugpageurl = get_option('siteurl').'/wp-admin/options-general.php?page=easy-admin-color-schemes';
$eacs_domain = 'EasyAdminColorSchemes';
load_plugin_textdomain($eacs_domain, 'wp-content/plugins/easy-admin-color-schemes');
add_action('init', 'eacs_init');

function eacs_init() {
	add_action('admin_init', 'eacs_css_files');

	if(function_exists('current_user_can') && current_user_can('manage_options')) {
		add_action('admin_menu', 'eacs_add_settings_page');

		if($_GET['page'] == 'easy-admin-color-schemes' || $_GET['page'] == 'easy-admin-color-schemes.php') {
			add_action('admin_init', 'eacs_process_request');
			add_action('admin_head', 'eacs_admin_head');
			wp_enqueue_script('colorpicker');
		}
	}
}

function eacs_error($code = 0) {
	if(preg_match('/^(1|2|3|4|5|6|9|10|11|12|13|14|15|16|17|18|19|20|21|22|23|24|33)$/', $code)) {
		$msg = __('A fatal error occurred while trying to interact with needed files and directories. This could mean your server is not configured properly and/or the files and directories are not writable&hellip;', $eacs_domain);
	} elseif(preg_match('/^(25|27|32)$/', $code)) {
		$msg = __('The specified action/method is invalid!', $eacs_domain);
	} elseif(preg_match('/^(7|8|26|29|31)$/', $code)) {
		$msg = __('The specified admin color scheme does not exist!', $eacs_domain);
	} elseif($code === 28) {
		$msg = __('A fatal error occurred while trying to set the current admin color scheme on the specified profile.', $eacs_domain);
	} elseif($code === 30) {
		$msg = __('A fatal error occurred while trying to delete the specified admin color scheme.', $eacs_domain);
	} else {
		$msg = __('An unknown fatal error occurred!', $eacs_domain);
		$code = 0;
	}

	wp_die(sprintf(__('<big>Fatal Error!</big><br />%1$s<br /><code>&mdash; Easy Admin Color Schemes [Error Code: %2$d]</code>', $eacs_domain), $msg, $code));
}

function eacs_process_request() {
	if(isset($_POST['eacsubmit']) || isset($_POST['eacsiesubmit']) || isset($_GET['eacsaction']) && $_GET['eacsaction'] != 'edit' && $_GET['eacsaction'] != 'export') {
		global $wpdb, $_wp_admin_css_colors, $userdata, $plugpageurl;
		get_currentuserinfo();
		if(eacs_css_files() === false) eacs_error(1); else $cssfiles = eacs_css_files();

		function isempty($var) {
			$var = trim($var);
			if(!isset($var) || empty($var)) return true; else return false;
		}

		if(isset($_POST['eacsubmit']) || isset($_POST['eacsiesubmit'])) {
			$schemedir = ABSPATH.PLUGINDIR.'/easy-admin-color-schemes/schemes/';
			$action = trim($_POST['eacsaction']);
			$ieaction = trim($_POST['eacsieaction']);
			$schemename = trim(stripslashes($_POST['schemename']));
			$cleanschemename = sanitize_title($schemename);
			$schemefile = $schemedir.$cleanschemename.'.css';
			$priclrtxt1 = trim($_POST['priclrtxt1']);
			$priclrtxt2 = trim($_POST['priclrtxt2']);
			$priclrtxt3 = trim($_POST['priclrtxt3']);
			$priclrtxt4 = trim($_POST['priclrtxt4']);
			$schemecss = trim(stripslashes($_POST['schemecss']));
			$exportmethod = trim($_POST['exportmethod']);
			$importcssfile = trim($_POST['importcssfile']);
			$importsizemax = trim($_POST['MAX_FILE_SIZE']);
			$exporthiddename = trim($_POST['eacsiename']);

			if(!is_writable($schemedir)) eacs_error(2);

			if($action == 'add' || $action == 'edit') {
				if(isempty($schemename) && isempty($priclrpck1) && isempty($priclrpck2) && isempty($priclrpck3) && isempty($priclrpck4) && isempty($schemecss)) {
					wp_redirect($plugpageurl.'&msg=7'); exit;
				}
			}

			if($action == 'add') {
				if(!file_exists($schemefile)) {
					if(!$cssfiles[$cleanschemename] && !$_wp_admin_css_colors[$cleanschemename]) {
						if(preg_match('/^[a-zA-Z0-9 \-\._\'"\?\!]{3,40}$/i', $schemename) && preg_match('/^\#[a-fA-F0-9]{6}$/i', $priclrtxt1) && preg_match('/^\#[a-fA-F0-9]{6}$/i', $priclrtxt2) && preg_match('/^\#[a-fA-F0-9]{6}$/i', $priclrtxt3) && preg_match('/^\#[a-fA-F0-9]{6}$/i', $priclrtxt4)) {
							$nfc = "/*\r\n".'Scheme Name: '.$schemename."\r\n".'Primary Colors: '.str_replace('#', '', $priclrtxt1).'|'.str_replace('#', '', $priclrtxt2).'|'.str_replace('#', '', $priclrtxt3).'|'.str_replace('#', '', $priclrtxt4)."\r\n*/\r\n".$schemecss;
							$sf = @fopen($schemefile, 'wb') or eacs_error(3);
							if(fwrite($sf, $nfc) === false) eacs_error(4);
							@fclose($sf);
							wp_redirect($plugpageurl.'&msg=1'); exit;
						} else {
							wp_redirect($plugpageurl.'&msg=9'); exit;
						}
					} else {
						wp_redirect($plugpageurl.'&msg=4'); exit;
					}
				} else {
					wp_redirect($plugpageurl.'&msg=4'); exit;
				}
			} elseif($action == 'edit') {
				$hiddename = trim($_POST['eacsname']);
				$schemefilebase = basename($cssfiles[$hiddename]['file']);
				$schemefile = $schemedir.$schemefilebase;
				if(file_exists($schemefile)) {
					if($cssfiles[$hiddename]) {
						if(preg_match('/^\#[a-fA-F0-9]{6}$/i', $priclrtxt1) && preg_match('/^\#[a-fA-F0-9]{6}$/i', $priclrtxt2) && preg_match('/^\#[a-fA-F0-9]{6}$/i', $priclrtxt3) && preg_match('/^\#[a-fA-F0-9]{6}$/i', $priclrtxt4)) {
							$nfc = "/*\r\n".'Scheme Name: '.$cssfiles[$hiddename]['name']."\r\n".'Primary Colors: '.str_replace('#', '', $priclrtxt1).'|'.str_replace('#', '', $priclrtxt2).'|'.str_replace('#', '', $priclrtxt3).'|'.str_replace('#', '', $priclrtxt4)."\r\n*/\r\n".$schemecss;
							$sf = @fopen($schemefile, 'wb') or eacs_error(5);
							if(fwrite($sf, $nfc) === false) eacs_error(6);
							@fclose($sf);
							wp_redirect($plugpageurl.'&eacsaction=edit&eacsid='.$hiddename.'&msg=2'); exit;
						} else {
							wp_redirect($plugpageurl.'&msg=9'); exit;
						}
					} else {
						eacs_error(7);
					}
				} else {
					eacs_error(8);
				}
			} elseif($ieaction == 'import') {
				if(!empty($_FILES['importcssfile']) && $_FILES['importcssfile']['error'] == 0) {
					$filename = basename($_FILES['importcssfile']['name']);
					$filecontent = @file_get_contents($_FILES['importcssfile']['tmp_name']) or eacs_error(9);
					$ext = strtolower(substr($filename, strrpos($filename, '.') + 1));

					if($ext == 'css' && $_FILES['importcssfile']['type'] == 'text/css' && $_FILES['importcssfile']['size'] < 2097152) {
						$newname = $schemedir.$filename;

						if(!file_exists($newname)) {
							if(preg_match('/^\/\*[\n\r]+Scheme Name: ([a-zA-Z0-9 \-\._\'"\?\!]{3,40})[\n\r]+Primary Colors: ([a-fA-F0-9]{6})[\|]([a-fA-F0-9]{6})[\|]([a-fA-F0-9]{6})[\|]([a-fA-F0-9]{6})[\n\r]+\*\/(.+)$/is', $filecontent)) {
								if(move_uploaded_file($_FILES['importcssfile']['tmp_name'], $newname)) {
									wp_redirect($plugpageurl.'&msg=10'); exit;
								} else {
									wp_redirect($plugpageurl.'&msg=11'); exit;
								}
							} else {
								wp_redirect($plugpageurl.'&msg=12'); exit;
							}
						} else {
							wp_redirect($plugpageurl.'&msg=13'); exit;
						}
					} else {
						wp_redirect($plugpageurl.'&msg=14'); exit;
					}
				} else {
					wp_redirect($plugpageurl.'&msg=15'); exit;
				}
			} elseif($ieaction == 'export') {
				if($cssfiles[$exporthiddename]) {
					@clearstatcache();
					$current_scheme = $cssfiles[$exporthiddename];
					$cssname = ltrim(basename($current_scheme['file'], '.css'), 'eacs_');
					$cssfilebase = basename($current_scheme['file']);
					$cssfile = $schemedir.$cssfilebase;
					$filesize = @filesize($cssfile) or eacs_error(10);
					$filecontent = trim($current_scheme['content']);

					if($exportmethod == 'method1') {
						header('Pragma: public');
						header('Expires: 0');
						header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
						header('Cache-Control: private', false);
						header('Content-Transfer-Encoding: binary');
						header('Content-Type: text/css');
						header('Content-Length: '.$filesize);
						header('Content-Disposition: attachment; filename="'.$cssfilebase.'";');
						@readfile($cssfile) or eacs_error(12);
						exit;
					} elseif($exportmethod == 'method2') {
						if(!empty($_ENV['TMP'])) $tempdir = $_ENV['TMP']; elseif(!empty($_ENV['TMPDIR'])) $tempdir = $_ENV['TMPDIR']; elseif(!empty($_ENV['TEMP'])) $tempdir = $_ENV['TEMP']; else $tempdir = dirname(tempnam('', 'na'));
						if(empty($tempdir)) eacs_error(13);
						if(is_writable($tempdir) == false) eacs_error(14);
						$tempdir = rtrim($tempdir, '/'); $tempdir .= '/';
						$tmpdir = $tempdir.'EACS'.rand(10000, 99999).'/';
						@mkdir($tmpdir) or eacs_error(15);
						$tmpcssfile = $tmpdir.'eacs_'.$cssname.'.css';
						$tmpphpfile = $tmpdir.'eacs_'.$cssname.'.php';
						$tmpzipfile = $tmpdir.'eacs_'.$cssname.'.zip';
						@chdir($tmpdir) or eacs_error(16);
						$opentmpcssfile = @fopen($tmpcssfile, 'w+') or eacs_error(17);
						$opentmpphpfile = @fopen($tmpphpfile, 'w+') or eacs_error(18);
						$parsedcssfile = "/* This CSS file was generated by the Easy Admin Color Schemes plugin for WordPress <http://www.jamesdimick.com/creations/easy-admin-color-schemes/> */\n\n".$filecontent;
						$parsedphpfile = sprintf('<?php'."\n".'/*'."\n".'Plugin Name: Admin Color Scheme: %1$s'."\n".'Description: This plugin adds the <strong>%1$s</strong> admin color scheme. This plugin was generated by the <a href="http://www.jamesdimick.com/creations/easy-admin-color-schemes/">Easy Admin Color Schemes</a> plugin for WordPress.'."\n".'*/'."\n\n".'add_action(\'admin_init\', \'eacs_admin_color_scheme_%3$s\');'."\n\n".'function eacs_admin_color_scheme_%3$s() {'."\n\t".'$plugin_url = get_option(\'siteurl\').\'/wp-content/plugins/\'.plugin_basename(dirname(__FILE__));'."\n\t".'wp_admin_css_color(\'%2$s\', \'%1$s\', $plugin_url.\'/%4$s.css\', array(\'%5$s\', \'%6$s\', \'%7$s\', \'%8$s\'));'."\n".'}'."\n".'?>', $current_scheme['name'], sanitize_title($current_scheme['name']), str_replace('-', '', sanitize_title($current_scheme['name'])), 'eacs_'.$cssname, $current_scheme['color1'], $current_scheme['color2'], $current_scheme['color3'], $current_scheme['color4']);
						$writetmpcssfile = @fwrite($opentmpcssfile, $parsedcssfile) or eacs_error(19);
						$writetmpphpfile = @fwrite($opentmpphpfile, $parsedphpfile) or eacs_error(20);
						if(!function_exists('PclZip')) require_once(ABSPATH.'wp-admin/includes/class-pclzip.php');
						$archive = new PclZip('eacs_'.$cssname.'.zip');
						$create = $archive->create($tmpcssfile.','.$tmpphpfile, PCLZIP_OPT_REMOVE_ALL_PATH, PCLZIP_OPT_ADD_PATH, 'eacs_'.$cssname);
						if($create == 0) eacs_error(21);
						$newzipfile = @fopen($tmpzipfile, 'r') or eacs_error(22);
						$newzipfilesize = @filesize($tmpzipfile) or eacs_error(23);
						header('Pragma: public');
						header('Expires: 0');
						header('Cache-Control: must-revalidate, post-check=0, pre-check=0');
						header('Cache-Control: private', false);
						header('Content-Transfer-Encoding: binary');
						header('Content-Type: application/zip');
						header('Content-Length: '.$newzipfilesize);
						header('Content-Disposition: attachment; filename="eacs_'.$cssname.'.zip";');
						@readfile($tmpzipfile) or eacs_error(24);
						@unlink($tmpcssfile); @unlink($tmpphpfile); @unlink($tmpzipfile); @rmdir($tmpdir);
						exit;
					} else {
						eacs_error(25);
					}
				} else {
					eacs_error(26);
				}
			} else {
				eacs_error(27);
			}
		} elseif($_GET['eacsaction'] == 'use') {
			if($_wp_admin_css_colors[$_GET['eacsid']]) {
				if(update_user_option($userdata->ID, 'admin_color', $_GET['eacsid'], true)) {
					wp_redirect($plugpageurl.'&msg=8'); exit;
				} else {
					eacs_error(28);
				}
			} else {
				eacs_error(29);
			}
		} elseif($_GET['eacsaction'] == 'delete') {
			if($cssfiles[$_GET['eacsid']]) {
				@unlink($cssfiles[$_GET['eacsid']]['file']) or eacs_error(30);
				wp_redirect($plugpageurl.'&msg=3'); exit;
			} else {
				eacs_error(31);
			}
		} else {
			eacs_error(32);
		}
	}
}

function eacs_css_files() {
	if(is_user_logged_in()) {
		global $_wp_admin_css_colors;
		@clearstatcache();
		$scheme_dir = ABSPATH.PLUGINDIR.'/easy-admin-color-schemes/schemes/';
		$dir = @opendir($scheme_dir);

		while($file = readdir($dir)) {
			if($file != '.' || $file != '..') {
				if(preg_match('/^(.+)\.css$/i', $file)) {
					$file_contents = @file_get_contents($scheme_dir.$file);

					if(preg_match('/^\/\*[\n\r]+Scheme Name: ([a-zA-Z0-9 \-\._\'"\?\!]{3,40})[\n\r]+Primary Colors: ([a-fA-F0-9]{6})[\|]([a-fA-F0-9]{6})[\|]([a-fA-F0-9]{6})[\|]([a-fA-F0-9]{6})[\n\r]+\*\/(.+)$/is', $file_contents, $matches)) {
						$clean_title = sanitize_title(trim($matches[1]));

						if(!$_wp_admin_css_colors[$clean_title]) {
							wp_admin_css_color($clean_title, $matches[1], get_option('siteurl').'/'.PLUGINDIR.'/easy-admin-color-schemes/schemes/'.$file, array('#'.$matches[2], '#'.$matches[3], '#'.$matches[4], '#'.$matches[5]));
						}

						if(!$result[$clean_title]) {
							$result[$clean_title] = array('name' => $matches[1], 'file' => $scheme_dir.$file, 'color1' => '#'.$matches[2], 'color2' => '#'.$matches[3], 'color3' => '#'.$matches[4], 'color4' => '#'.$matches[5], 'content' => $matches[6]);
						}
					}
				}
			}
		}

		@closedir($dir);
		$result = (isset($result) && !empty($result) && $result != '') ? $result : false;
		return($result);
	} else {
		return(false);
	}
}

function eacs_admin_head() {
	global $_wp_admin_css_colors; ?>
	<link rel="stylesheet" type="text/css" href="<?php echo(get_option('siteurl').'/'.PLUGINDIR.'/easy-admin-color-schemes/easy-admin-color-schemes.css') ?>" />
	<script type="text/javascript">
	// <![CDATA[
	var cp = new ColorPicker();
	function deleteConfirm(sn) {
		return confirm('<?php _e('Are you sure you want to delete the', $eacs_domain) ?> \"' + sn + '\" <?php _e('scheme?', $eacs_domain) ?>\n<?php _e('This action cannot be undone!', $eacs_domain) ?>');
	}
	function validate_form(thisform) {
		var namePatt = new RegExp('^[a-zA-Z0-9 \-\._\'"\?\!]{3,40}$', 'i');
		var colorPatt = new RegExp('^\#[a-fA-F0-9]{6}$', 'i');
		with(thisform) {
			if(schemename.value == null || schemename.value == '' || namePatt.test(schemename.value) == false) {
				schemename.focus(); alert('<?php _e('The scheme name is empty or invalid!', $eacs_domain) ?>'); return false;
			} else if(priclrtxt1.value == null || priclrtxt1.value == '' || colorPatt.test(priclrtxt1.value) == false) {
				priclrtxt1.focus(); alert('<?php _e('The first primary color is empty or invalid!', $eacs_domain) ?>'); return false;
			} else if(priclrtxt2.value == null || priclrtxt2.value == '' || colorPatt.test(priclrtxt2.value) == false) {
				priclrtxt2.focus(); alert('<?php _e('The second primary color is empty or invalid!', $eacs_domain) ?>'); return false;
			} else if(priclrtxt3.value == null || priclrtxt3.value == '' || colorPatt.test(priclrtxt3.value) == false) {
				priclrtxt3.focus(); alert('<?php _e('The third primary color is empty or invalid!', $eacs_domain) ?>'); return false;
			} else if(priclrtxt4.value == null || priclrtxt4.value == '' || colorPatt.test(priclrtxt4.value) == false) {
				priclrtxt4.focus(); alert('<?php _e('The fourth primary color is empty or invalid!', $eacs_domain) ?>'); return false;
			} else if(schemecss.value == null || schemecss.value == '') {
				schemecss.focus(); alert('<?php _e('The scheme CSS is empty or invalid!', $eacs_domain) ?>'); return false;
			} else if(importcssfile.value == null || importcssfile.value == '') {
				importcssfile.focus(); alert('<?php _e('The upload file is empty or invalid!', $eacs_domain) ?>'); return false;
			}
		}
	}
	function validate_form_2(thisform) {
		with(thisform) {
			if(importcssfile.value == null || importcssfile.value == '') {
				importcssfile.focus(); alert('<?php _e('The scheme CSS file is empty or invalid!', $eacs_domain) ?>'); return false;
			}
		}
	}
	function PopupWindow_populate(contents) {
		contents += '<p style="text-align:center;margin-top:10px"><input type="button" value="<?php _e('Close Color Picker', $eacs_domain) ?>" onclick="cp.hidePopup(\'prettyplease\')" class="button-secondary" /><' + '/p>';
		this.contents = contents;
		this.populated = false;
	}
	function colorSelect(t, p) {
		var t = document.getElementById(t);
		if(cp.p == p && document.getElementById(cp.divName).style.visibility != 'hidden') {
			cp.hidePopup('prettyplease');
		} else {
			cp.p = p;
			cp.select(t, p);
		}
	}
	function insertColorPickerButtons() {
		var pckbtn1 = document.createElement('input'); pckbtn1.setAttribute('title', '<?php _e('Click to choose a color', $eacs_domain) ?>'); pckbtn1.setAttribute('class', 'button-secondary'); pckbtn1.setAttribute('onclick', "colorSelect('priclrtxt1','priclrpck1');return false;"); pckbtn1.setAttribute('value', '>'); pckbtn1.setAttribute('id', 'priclrpck1'); pckbtn1.setAttribute('name', 'priclrpck1'); pckbtn1.setAttribute('type', 'button');
		var pckbtn2 = document.createElement('input'); pckbtn2.setAttribute('title', '<?php _e('Click to choose a color', $eacs_domain) ?>'); pckbtn2.setAttribute('class', 'button-secondary'); pckbtn2.setAttribute('onclick', "colorSelect('priclrtxt2','priclrpck2');return false;"); pckbtn2.setAttribute('value', '>'); pckbtn2.setAttribute('id', 'priclrpck2'); pckbtn2.setAttribute('name', 'priclrpck2'); pckbtn2.setAttribute('type', 'button');
		var pckbtn3 = document.createElement('input'); pckbtn3.setAttribute('title', '<?php _e('Click to choose a color', $eacs_domain) ?>'); pckbtn3.setAttribute('class', 'button-secondary'); pckbtn3.setAttribute('onclick', "colorSelect('priclrtxt3','priclrpck3');return false;"); pckbtn3.setAttribute('value', '>'); pckbtn3.setAttribute('id', 'priclrpck3'); pckbtn3.setAttribute('name', 'priclrpck3'); pckbtn3.setAttribute('type', 'button');
		var pckbtn4 = document.createElement('input'); pckbtn4.setAttribute('title', '<?php _e('Click to choose a color', $eacs_domain) ?>'); pckbtn4.setAttribute('class', 'button-secondary'); pckbtn4.setAttribute('onclick', "colorSelect('priclrtxt4','priclrpck4');return false;"); pckbtn4.setAttribute('value', '>'); pckbtn4.setAttribute('id', 'priclrpck4'); pckbtn4.setAttribute('name', 'priclrpck4'); pckbtn4.setAttribute('type', 'button');
		document.getElementById('priclrs').insertBefore(pckbtn1, document.getElementById('priclrtxt1').nextSibling);
		document.getElementById('priclrs').insertBefore(pckbtn2, document.getElementById('priclrtxt2').nextSibling);
		document.getElementById('priclrs').insertBefore(pckbtn3, document.getElementById('priclrtxt3').nextSibling);
		document.getElementById('priclrs').insertBefore(pckbtn4, document.getElementById('priclrtxt4').nextSibling);
	}
	function insertColorPickerDiv() {
		var thediv = '<div id="colorPickerDiv">&nbsp;<' + '/div>';
		document.getElementById('priclrs').innerHTML += thediv;
	}
	function insertPreview() {
		var optbl = document.getElementById('schemeformtable').getElementsByTagName('tbody')[0];
		var newtr = document.createElement('tr');
		var newth = document.createElement('th');
		optbl.appendChild(newtr);
		var thetr = optbl.getElementsByTagName('tr')[3];
		thetr.vAlign = 'top';
		thetr.appendChild(newth);
		var theth = thetr.getElementsByTagName('th')[0];
		theth.className = 'th-full';
		theth.colSpan = 2;
		theth.scope = 'row';
		theth.innerHTML = '<span style="display:block;float:left;margin-bottom:10px"><?php _e('Live Preview', $eacs_domain) ?><' + '/span><input type="button" name="schemeprevupd" id="schemeprevupd" value="<?php _e('Update', $eacs_domain) ?>" onclick="updatePreview();return false;" class="button-secondary" title="<?php _e('Click to update the live preview below', $eacs_domain) ?>" /><iframe name="schemepreview" id="schemepreview" src="<?php echo(get_option('siteurl').'/wp-admin/') ?>" onload="updatePreview();return false;"><?php _e('Scheme Preview', $eacs_domain) ?><' + '/iframe>';
	}
	function updatePreview() {
		var resetCSS = '* { color: #000 !important; background-color: #fff !important; }';
		var innerCSS = document.getElementById('schemecss').value;
		with(document.getElementById('schemepreview').contentWindow) {
			var h = document.getElementsByTagName('head'); if(!h.length)return;
			for(i = 0; i < document.styleSheets.length; i++) {
				if(document.styleSheets[i].href != null) {
					if(document.styleSheets[i].href.indexOf('<?php echo($_wp_admin_css_colors[get_user_option('admin_color')]->url) ?>') > -1) {
						document.styleSheets[i].disabled = true;
					}
				}
			}
			var newStyleSheet = document.getElementById('eacspreviewstyles');
			if(!newStyleSheet) {
				newStyleSheet = document.createElement('style');
				newStyleSheet.type = 'text/css';
				newStyleSheet.id = 'eacspreviewstyles';
				h[0].appendChild(newStyleSheet);
			}
			try {
				if(!innerCSS.length)return newStyleSheet.styleSheet.cssText = '';
				newStyleSheet.styleSheet.cssText = innerCSS;
			} catch(e) {
				while(newStyleSheet.firstChild)newStyleSheet.removeChild(newStyleSheet.firstChild);
				newStyleSheet.appendChild(document.createTextNode(innerCSS));
			}
		}
	}
	window.onload = function() { insertColorPickerButtons(); insertColorPickerDiv(); insertPreview(); };
	// ]]>
	</script>
<?php }

function eacs_add_settings_page() {
	function eacs_settings_page() {
		global $_wp_admin_css_colors, $plugpageurl;
		$cssfiles = eacs_css_files();
		$default_scheme = @file_get_contents(ABSPATH.'/wp-admin/css/colors-fresh.css') or eacs_error(33);
		$current_scheme = $cssfiles[$_GET['eacsid']];
		$current_scheme_cleaname = sanitize_title($current_scheme['name']);
		$scheme_header = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? __('Edit Color Scheme', $eacs_domain) : __('Create a Color Scheme', $eacs_domain);
		$scheme_headdesc = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? __('Use the form below to edit the currently selected color scheme.', $eacs_domain) : __('You can fill out the simple form below to create a new scheme.', $eacs_domain);
		$scheme_name = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? $current_scheme['name'] : 'My Cool Scheme';
		$scheme_nameclr = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? '' : ' onfocus="if(this.value == \'My Cool Scheme\') { this.value=\'\'; }" onblur="if(this.value == \'\') { this.value=\'My Cool Scheme\'; }"';
		$scheme_namedis = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? ' disabled="disabled"' : '';
		$scheme_priclr1 = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? $current_scheme['color1'] : '#464646';
		$scheme_priclr2 = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? $current_scheme['color2'] : '#CEE1EF';
		$scheme_priclr3 = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? $current_scheme['color3'] : '#D54E21';
		$scheme_priclr4 = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? $current_scheme['color4'] : '#2683AE';
		$scheme_css = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? $current_scheme['content'] : $default_scheme;
		$scheme_hiddename = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? '<input type="hidden" name="eacsname" id="eacsname" value="'.sanitize_title($scheme_name).'" />' : '';
		$scheme_action = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? 'edit' : 'add';
		$scheme_submitval = ($_GET['eacsaction'] == 'edit' && $current_scheme) ? __('Save Scheme', $eacs_domain) : __('Create Scheme', $eacs_domain);
		$importexport_header = ($_GET['eacsaction'] == 'export' && $current_scheme) ? __('Export Color Scheme', $eacs_domain) : __('Import a Color Scheme', $eacs_domain);
		$importexport_headdesc = ($_GET['eacsaction'] == 'export' && $current_scheme) ? __('Use the options below to export the selected color scheme.', $eacs_domain) : __('Use the form below to import a pre-existing color scheme for use with your WordPress admin interface.', $eacs_domain);
		$import_enctype = ($_GET['eacsaction'] == 'export' && $current_scheme) ? '' : ' enctype="multipart/form-data"';
		$import_sizemax = ($_GET['eacsaction'] == 'export' && $current_scheme) ? '' : '<input type="hidden" name="MAX_FILE_SIZE" value="2097152" />';
		$export_hiddename = ($_GET['eacsaction'] == 'export' && $current_scheme) ? '<input type="hidden" name="eacsiename" id="eacsiename" value="'.$current_scheme_cleaname.'" />' : '';
		$importexport_action = ($_GET['eacsaction'] == 'export' && $current_scheme) ? 'export' : 'import';
		$importexport_submitval = ($_GET['eacsaction'] == 'export' && $current_scheme) ? __('Export Scheme', $eacs_domain) : __('Import Scheme', $eacs_domain);

		function stream_last_modified($url) {
			if(!($fp = @fopen($url, 'r'))) { return(NULL); }
			$meta = stream_get_meta_data($fp);
			for($j = 0; isset($meta['wrapper_data'][$j]); $j++) {
				if(strstr(strtolower($meta['wrapper_data'][$j]), 'last-modified')) {
					$modtime = substr($meta['wrapper_data'][$j], 15);
					break;
				}
			}
			$result = (isset($modtime)) ? strtotime($modtime) : NULL;
			fclose($fp); return($result);
		}

		if(isset($_GET['msg'])) {
			if($_GET['msg'] == '1') { ?>
				<div id="eacsmsg" class="updated fade"><p><strong><?php _e('Your new color scheme has been added!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '2') { ?>
				<div id="eacsmsg" class="updated fade"><p><strong><?php _e('Your changes to the color scheme have been saved!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '3') { ?>
				<div id="eacsmsg" class="updated fade"><p><strong><?php _e('You have successfully deleted the color scheme!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '4') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('The color scheme you were trying to add already exists!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '5') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('The provided scheme is not a valid color scheme!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '6') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('The action provided is incorrect!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '7') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('Your submission had missing required items!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '8') { ?>
				<div id="eacsmsg" class="updated fade"><p><strong><?php _e('Your current color scheme has been changed!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '9') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('Your submission had invalid required items!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '10') { ?>
				<div id="eacsmsg" class="updated fade"><p><strong><?php _e('Your color scheme has been successfully imported!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '11') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('The color scheme could not be imported!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '12') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('The color scheme you were trying to import does not have the proper header!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '13') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('The color scheme you were trying to import already exists!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '14') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('Imported color schemes can only be .CSS files smaller than 2MB!', $eacs_domain) ?></strong></p></div>
			<?php } elseif($_GET['msg'] == '15') { ?>
				<div id="eacsmsg" class="error"><p><strong><?php _e('No file was uploaded!', $eacs_domain) ?></strong></p></div>
			<?php }
		} ?>
		<div class="wrap">
			<h2 id="all-schemes"><?php _e('Color Schemes', $eacs_domain) ?></h2>
			<div class="section">
				<p><?php printf(__('Below you will find a list of all currently existing color schemes. You can add more by going down to the <a href="%1$s" title="Create a color scheme">Create a Color Scheme</a> area. You can also use the <a href="%2$s" title="Import a color scheme">Import a Color Scheme</a> area to import a pre-existing color scheme.', $eacs_domain), $plugpageurl.'#scheme-input', $plugpageurl.'#import-export') ?></p>
				<table class="widefat" id="the-scheme-list">
					<thead>
						<tr>
							<th scope="col"><?php _e('Scheme', $eacs_domain) ?></th>
							<th scope="col" style="width:20%"><?php _e('Last Modified', $eacs_domain) ?></th>
							<th scope="col" style="text-align:right;width:20%"><?php _e('Actions', $eacs_domain) ?></th>
						</tr>
					</thead>

					<tbody>
						<?php if($_wp_admin_css_colors) {
							$current_user_admin_color = (get_user_option('admin_color')) ? get_user_option('admin_color') : 'fresh';

							foreach($_wp_admin_css_colors as $color => $color_info) {
								$schememodtime = stream_last_modified($color_info->url);
								$schememodate = ($schememodtime) ? date('Y/m/d \a\\t g:iA', $schememodtime) : 'Unknown'; ?>
								<tr>
									<td class="eacs-schemes-colors"><?php foreach($color_info->colors as $html_color) { ?><span style="background-color:<?php echo($html_color) ?>">&nbsp;</span><?php } ?><strong><?php echo($color_info->name) ?></strong></td>
									<td style="width:20%"><?php _e($schememodate, $eacs_domain) ?></td>
									<td style="text-align:right;width:20%"><?php $use_link = ($color != $current_user_admin_color) ? '<a href="'.$plugpageurl.'&amp;eacsaction=use&amp;eacsid='.$color.'" title="'.__('Use this color scheme now', $eacs_domain).'">'.__('Use', $eacs_domain).'</a>' : '<span class="nonessential">'.__('Use', $eacs_domain).'</span>';

									if($cssfiles[$color]) {
										$edit_link = ($_GET['eacsaction'] == 'edit' && $_GET['eacsid'] == $color) ? '<span class="nonessential">'.__('Edit', $eacs_domain).'</span>' : '<a href="'.$plugpageurl.'&amp;eacsaction=edit&amp;eacsid='.$color.'#scheme-input" title="'.__('Edit this color scheme', $eacs_domain).'">'.__('Edit', $eacs_domain).'</a>';
										$export_link = ($_GET['eacsaction'] == 'export' && $_GET['eacsid'] == $color) ? '<span class="nonessential">'.__('Export', $eacs_domain).'</span>' : '<a href="'.$plugpageurl.'&amp;eacsaction=export&amp;eacsid='.$color.'#import-export" title="'.__('Export this color scheme', $eacs_domain).'">'.__('Export', $eacs_domain).'</a>';
										$delete_link = '<a href="'.$plugpageurl.'&amp;eacsaction=delete&amp;eacsid='.$color.'" title="'.__('Delete this color scheme', $eacs_domain).'" class="delete" onclick="return deleteConfirm(\''.addslashes($color_info->name).'\');">'.__('Delete', $eacs_domain).'</a>';
									} else {
										$edit_link = '<span class="nonessential">'.__('Edit', $eacs_domain).'</span>';
										$export_link = '<span class="nonessential">'.__('Export', $eacs_domain).'</span>';
										$delete_link = '<span class="nonessential">'.__('Delete', $eacs_domain).'</span>';
									}

									echo($use_link.' | '.$edit_link.' | '.$export_link.' | '.$delete_link); ?></td>
								</tr>
							<?php }
						} else { ?>
							<tr>
								<td colspan="3"><strong><?php _e('There are currently no color schemes! Add some below&hellip;', $eacs_domain) ?></strong></td>
							</tr>
						<?php } ?>
					</tbody>
				</table>
			</div>


			<h2 id="scheme-input" style="margin-top:2em"><?php echo($scheme_header) ?></h2>
			<div class="section">
				<p><?php echo($scheme_headdesc) ?> <?php _e('Also, if JavaScript is enabled in your browser, there is a live preview function.', $eacs_domain) ?></p>
				<form method="post" action="<?php echo($plugpageurl) ?>" onsubmit="return validate_form(this);">
					<table id="schemeformtable" class="form-table">
						<tbody>
							<tr valign="top">
								<th scope="row"><label for="schemename"><?php _e('Scheme Name', $eacs_domain) ?></label></th>
								<td><input type="text" name="schemename" id="schemename" value="<?php echo($scheme_name) ?>" size="50" maxlength="40"<?php echo($scheme_namedis.$scheme_nameclr) ?> /></td>
							</tr>

							<tr valign="top">
								<th scope="row"><label for="priclrtxt1"><?php _e('Primary Colors', $eacs_domain) ?></label></th>
								<td id="priclrs"><input type="text" name="priclrtxt1" id="priclrtxt1" value="<?php echo($scheme_priclr1) ?>" size="8" maxlength="7" class="code" />&emsp;<input type="text" name="priclrtxt2" id="priclrtxt2" value="<?php echo($scheme_priclr2) ?>" size="8" maxlength="7" class="code" />&emsp;<input type="text" name="priclrtxt3" id="priclrtxt3" value="<?php echo($scheme_priclr3) ?>" size="8" maxlength="7" class="code" />&emsp;<input type="text" name="priclrtxt4" id="priclrtxt4" value="<?php echo($scheme_priclr4) ?>" size="8" maxlength="7" class="code" /><p><?php _e('<strong>Please keep in mind:</strong> The primary colors only serve as a sort of preview for the color scheme. Changing these colors have no effect on any part of the actual color scheme. You must edit the CSS below to actually change anything.', $eacs_domain) ?></p></td>
							</tr>

							<tr valign="top">
								<th scope="row"><label for="schemecss"><?php _e('Scheme CSS', $eacs_domain) ?></label></th>
								<td><textarea name="schemecss" id="schemecss" cols="50" rows="20" class="code"><?php echo($scheme_css) ?></textarea></td>
							</tr>
						</tbody>
					</table>
					<p class="submit"><?php echo($scheme_hiddename) ?><input type="hidden" name="eacsaction" id="eacsaction" value="<?php echo($scheme_action) ?>" /><input type="submit" name="eacsubmit" id="eacsubmit" value="<?php echo($scheme_submitval) ?>" class="button" onclick="cp.hidePopup('prettyplease');" /></p>
				</form>
			</div>


			<h2 id="import-export" style="margin-top:2em"><?php echo($importexport_header) ?></h2>
			<div class="section">
				<p><?php echo($importexport_headdesc) ?></p>
				<form method="post" action="<?php echo(get_option('siteurl')) ?>/wp-admin/options-general.php?page=easy-admin-color-schemes"<?php echo($import_enctype) ?> onsubmit="return validate_form_2(this);">
					<table id="schemeietable" class="form-table">
						<tbody>
							<?php if($_GET['eacsaction'] == 'export' && $current_scheme) { ?>
								<tr valign="top">
									<th scope="row"><?php _e('Current Scheme', $eacs_domain) ?></th>
									<td><?php _e('The current color scheme selected for export is:', $eacs_domain) ?> <strong><?php echo($current_scheme['name']) ?></strong></td>
								</tr>

								<tr valign="top">
									<th scope="row"><label for="exportmethod"><?php _e('Export Method', $eacs_domain) ?></label></th>
									<td><p><input type="radio" name="exportmethod" id="exportmethod1" value="method1" checked="checked" /> <strong><label for="exportmethod1">EACS CSS File</label></strong> - <?php _e('This method exports the currently selected color scheme as a special CSS file with special header text at the beginning of the file which allows it to be imported back into this plugin.', $eacs_domain) ?></p><p><input type="radio" name="exportmethod" id="exportmethod2" value="method2" /> <strong><label for="exportmethod2">WordPress Plugin</label></strong> - <?php _e('This method exports the currently selected color scheme as its own WordPress plugin so it can be used by itself on any WordPress site without requiring the Easy Admin Color Schemes plugin.', $eacs_domain) ?></p></td>
								</tr>
							<?php } else { ?>
								<tr valign="top">
									<th scope="row"><label for="importcssfile"><?php _e('Scheme CSS File', $eacs_domain) ?></label></th>
									<td><input type="file" name="importcssfile" id="importcssfile" size="60" /><br /><small><?php _e('Please note, only files with the <strong>.CSS</strong> extension can be imported. Files bigger than <strong>2MB</strong> cannot be imported!<br />Imported files <strong>MUST</strong> have the proper header at the beginning of the file in order to be recognized by this plugin!<br /><br />A proper header looks like this:<br /><strong>/*<br />Scheme Name: Washedout<br />Primary Colors: 464646|DEDEDE|D54E21|6A6A6A<br />*/</strong>', $eacs_domain) ?></small></td>
								</tr>
							<?php } ?>
						</tbody>
					</table>
					<p class="submit"><?php echo($import_sizemax.$export_hiddename) ?><input type="hidden" name="eacsieaction" id="eacsieaction" value="<?php echo($importexport_action) ?>" /><input type="submit" name="eacsiesubmit" id="eacsiesubmit" value="<?php echo($importexport_submitval) ?>" class="button" /></p>
				</form>
			</div>


			<h2 id="more-info" style="margin-top:2em"><?php _e('More Information', $eacs_domain) ?></h2>
			<div class="section">
				<p><?php _e('This plugin allows you to manage the color schemes for the administration interface of this site. You can add, edit, and delete any scheme that is compatible with this plugin. The interface is fairly straightforward but, if you have any issues please <a href="http://www.jamesdimick.com/contact/">get in contact with the author</a>. Also, please report any bugs, errors, or just anything that you would like at <a href="http://www.jamesdimick.com/creations/easy-admin-color-schemes/">the official plugin page</a>. Finally, if you like this plugin and would like to help support it&rsquo;s development, you can <a href="http://www.jamesdimick.com/donate/">donate to the author</a>. Donations are <strong>very</strong> much appreciated and they will all go to making this plugin better!', $eacs_domain) ?></p>
			</div>
		</div>
	<?php }

	/* This does not work reliably enough yet. Commenting it out for now.
	function eacs_manage_schemes_link() {
		global $plugpageurl; ?>
		<p style="position:absolute;margin:14em 0 0 25px"><a href="<?php echo($plugpageurl) ?>" title="<?php _e('Click here to manage the Admin Color Schemes', $eacs_domain) ?>"><?php _e('Manage &raquo;', $eacs_domain) ?></a></p>
	<?php }*/

	add_submenu_page('options-general.php', __('Color Schemes', $eacs_domain), __('Color Schemes', $eacs_domain), 'manage_options', 'easy-admin-color-schemes', 'eacs_settings_page');
	/* This does not work reliably enough yet. Commenting it out for now.
	$currurlparse = parse_url($_SERVER['REQUEST_URI']);
	$currurl = basename($currurlparse['path']);
	if(preg_match('/^user\-edit\.php(.*)$/i', $currurl) || preg_match('/^profile\.php(.*)$/i', $currurl)) { add_action('admin_notices', 'eacs_manage_schemes_link', 5000000); }
	*/
}

function eacs_rightnow() {
	if(function_exists('get_user_option') && function_exists('current_user_can')) { ?>
		<p class="eacs-right-now"><?php printf(__('You are using the %1$s admin color scheme.', $eacs_domain), ucwords(get_user_option('admin_color'))); if(current_user_can('manage_options')) { ?> <a href="<?php echo(get_option('siteurl').'/wp-admin/options-general.php?page=easy-admin-color-schemes') ?>" class="rbutton"><?php _e('Change Scheme', $eacs_domain) ?></a><?php } ?></p>
	<?php }
}

add_action('rightnow_end', 'eacs_rightnow', 1);
?>
